<?php
set_time_limit  ( 180  ); 					// allow 3 minutes for this script to run!!
include('config.php'); 						// load WB configuration

// --------------------------------------------------------------------------------------------------------------------------------------------
// Configuration

$emailAddressToAlert = SERVER_EMAIL;   		// Alternative something like:  $emailAddressToAlert = 'ruud.eisinga@gmail.com';
$scanPath = WB_PATH;						// Scan your complete WebsiteBaker webspace
$noScanDirs = "/cache,/awstats,/stats";		// Directories with these names will not be scanned (i.e. awstats generates files daily)
$noScanExt = ".jpg,.png,.gif";				// Files with these extensions will not be scanned (i.e. images)
$checkmd5 = true; 							// A better way of detecting all changes, but with more load and slower on your server.
$silent = false;							// When set true, the script will not output anything.

// End configuration, do not change anything below this line!
// --------------------------------------------------------------------------------------------------------------------------------------------

new croncheck($emailAddressToAlert, $scanPath, $checkmd5, $noScanDirs, $noScanExt, $silent);
class croncheck {

	private $aPrevious = array();
	private $aChanged = array();
	private $aNew = array();
	private $aCurrent = array();
	private $md5result;
	private $md5calc;
	private $startScan;
	private $noScanDirs;
	private $noScanExt;
	private $emailAddress;
	private $resultMessage = "";
	private $silent;
	
	function __construct($emailAddress = SERVER_EMAIL, $path = WB_PATH, $checkmd5 = true, $noScanDirs = "", $noScanExt = "", $silent = true) {
		$this->startScan = microtime(true);		
		if(!$this->hasTable()) $this->createTable();
		$this->getLast();
		$this->noScanDirs = explode(",",$noScanDirs);
		$this->noScanExt = explode(",",$noScanExt);
		$this->emailAddress = $emailAddress;
		$this->silent = $silent;
		$this->scan($path,$checkmd5);
		$this->saveScan();
		$this->report();
		$this->cleanTable();
	}
	
	function getScantime() {
		$now = microtime(true);
		return $now - $this->startScan;
	}
	
	function hasTable() {
		global $database;
		if ($database->get_one("SHOW TABLES LIKE '".TABLE_PREFIX."croncheck';")) {
			return true;
		} else {
			return false;
		}
	}

	function dropTable() {
		global $database;
		$database->query("DROP TABLE IF EXISTS ".TABLE_PREFIX."croncheck");
	}
	
	function createTable() {
		global $database;
		$database->query("CREATE TABLE `".TABLE_PREFIX."croncheck` (
			`id` int(11) NOT NULL auto_increment,
			`timestamp` int(11) NOT NULL default 0,
			`md5` varchar(64) NOT NULL default '',
			`data` LONGTEXT NOT NULL default '',
			`added` LONGTEXT NOT NULL default '',
			`removed` LONGTEXT NOT NULL default '',
			`modified` LONGTEXT NOT NULL default '',
			PRIMARY KEY  (`id`)
			)"
		);
	}

	function cleanTable() {
		global $database;
		$database->query("DELETE FROM `".TABLE_PREFIX."croncheck` WHERE id NOT IN ( SELECT id FROM ( SELECT id FROM `".TABLE_PREFIX."croncheck` ORDER BY timestamp DESC LIMIT 5 ) foo );");
	}
	
	function getLast() {
		global $database;
		$query = $database->query("SELECT * from `".TABLE_PREFIX."croncheck` ORDER BY `id` DESC LIMIT 1");
		if($query && $query->numRows()) {
			$row = $query->fetchRow();
			$this->aPrevious = $this->unserialize($row['data']);
		}
	}
	
	function saveScan() {
		global $database;
		$save = false;
		if(!$this->aNew && !$this->aChanged && !$this->aPrevious) return;
		$database->query("INSERT INTO `".TABLE_PREFIX."croncheck` (`timestamp`,`md5`,`data`,`added`,`removed`,`modified`) VALUES (
			'".time()."',
			'".$this->md5calc."',
			'".serialize($this->aCurrent)."',
			'".serialize($this->aNew)."',
			'".serialize($this->aPrevious)."',
			'".serialize($this->aChanged)."')");
		if($database->is_error()) $this->resultMessage .= $database->get_error()."\n";
	}
	
	function unserialize($sObject) {
		$__ret = preg_replace_callback(
			'!s:(\d+):"(.*?)";!s',
			function($m){
				$len = strlen($m[2]);
				$result = "s:$len:\"{$m[2]}\";";
				return $result;

			},
			$sObject);
		return unserialize($__ret);
	}  
	
	function scan($path = '.',$checkmd5 = true) {
		$base = str_replace('\\','/',WB_PATH);
		$md = '';
		$file = new RecursiveIteratorIterator(new RecursiveDirectoryIterator($path));
		while($file->valid()) {
			if (!$file->isDot()) {
				$cf =  str_replace('\\','/',$file->getPathname());
				if ($this->strpos_array($cf,$this->noScanDirs) === false) {
					if ($this->strpos_array($cf,$this->noScanExt, true) === false) {
						$ft = $file->getMTime(); 
						if($checkmd5) $md = md5_file($cf);
						$curr = addslashes(str_replace($base,"",$cf));
						if(isset($this->aPrevious[$curr])) {
							$details = explode("|", $this->aPrevious[$curr]);
							if (($checkmd5 && $md != $details[1]) || $ft != $details[0]) {
								$this->aChanged[$curr] = $ft.'|'.$md;
							} 
							unset($this->aPrevious[$curr]);
						} elseif (!isset($this->aCurrent[$curr])) {
							$this->aNew[$curr] = $ft.'|'.$md;
						}
						$this->aCurrent[$curr] = $ft.'|'.$md;
					}
				}
			}
			$file->next();
		}
	}
	
	function strpos_array($haystack, $needles,$ext = false) {
		if ( is_array($needles) ) {
			foreach ($needles as $str) {
				if ( is_array($str) ) {
					$pos = strpos_array($haystack, $str);
				} else {
					if($ext) {
						$pos = strcasecmp(substr($haystack,-(strlen($str))),$str)==0;
					}else {
						$pos = stripos($haystack, $str);
					}
				}
				if ($pos !== FALSE) {
					return $pos;
				}
			}
		} else {
			return stripos($haystack, $needles);
		}
		return false;
	}
	
	function report() {
		if($this->aNew || $this->aChanged || $this->aPrevious) {
			$this->resultMessage .= "Something has changed on ".WB_URL."\n";
			if($this->aNew) {
				$this->resultMessage .= "\nNew files were added: \n\n";
				foreach ($this->aNew as $key => $value) {
					$details = explode("|",$value);
					$this->resultMessage .= $key .' -> ' . date(DATE_FORMAT.' '.TIME_FORMAT,$details[0]+TIMEZONE). "\n";
				}
			}
			
			if($this->aChanged) {
				$this->resultMessage .= "\nThese files are changed: \n\n";
				foreach ($this->aChanged as $key => $value) {
					$details = explode("|",$value);
					$this->resultMessage .= $key .' -> ' . date(DATE_FORMAT.' '.TIME_FORMAT,$details[0]+TIMEZONE). "\n";
				}
			}
			
			if($this->aPrevious) {
				$this->resultMessage .= "\nThese files have been removed: \n\n";
				foreach ($this->aPrevious as $key => $value) {
					$details = explode("|",$value);
					$this->resultMessage .= $key .' -> ' . date(DATE_FORMAT.' '.TIME_FORMAT,$details[0]+TIMEZONE). "\n";
				}
			}
			$this->resultMessage .= "\n\nThe script was running for ".$this->getScantime()." seconds";
			$headers = "From: CronCheck <".SERVER_EMAIL.">\r\n";
			$headers .= "Reply-To: ".$this->emailAddress."\r\n";
			$emailSubject = "IMPORTANT! Files in ".WB_URL." have changed";
			mail($this->emailAddress, $emailSubject, $this->resultMessage, $headers); 
		} else {
			$this->resultMessage = " - ALLOK - Nothing has changed.\n\nThe script was running for ".$this->getScantime()." seconds";
		}
		if(!$this->silent) echo '<pre>'.$this->resultMessage;
	}
}
